#include "graphics.h"
#include "map.h"
#include "chaos.h"
#include "player.h"
#include "enemy.h"
#include <math.h>

void corruptTile(int x, int y, float c)
{
    if(x<0 || y<0 || x >=90 || y>=40)
      return;
    c*=c;
    c*=c;
    c/=2;
    int c2=c*255;
    if(c2>0)
    {
        map[x][y].r^=rand()%(int)(c2*1.2);
        map[x][y].g^=rand()%(int)(c2*1.2);
        map[x][y].b^=rand()%(int)(c2*1.2);
        if(rand()%c2>115)
          map[x][y].c=' '+rand()%(16*6);
        addChange(x,y);
    }
}

void corruptInRange(int range)
{
    if(range<1)
      return;
    int minX=player.x-range,maxX=player.x+range;
    int minY=player.y-range,maxY=player.y+range;
    if(minX<0)
      minX=0;
    if(minY<0)
      minY=0;
    if(maxX>=90)
      maxX=89;
    if(maxY>=40)
      maxY=39;
    int i,j,r2=range*range,d;
    for(j=minY;j<=maxY;j++)
      for(i=minX;i<=maxX;i++)
      {
          d=(player.x-i)*(player.x-i)+(player.y-j)*(player.y-j);
          if(d<r2)
            corruptTile(i,j,1-(float)d/r2);
      }
}

void corrupt(int num)
{
    int i,r;
    if(num>0)
      corruptInRange(sqrt(num));
    for(i=0;i<num;i++)
    {
        r=((rand()%(16*12))*gameTex->width+(rand()%(16*8)))*4+rand()%4;
        gameTex->dat[r]^=1<<(rand()%8);
        gameTex->flags|=TEX_FLAG_DIRTY;
    }
    for(i=0;i<num/10;i++)
    {
        r=((rand()%(16*12))*fontTex->width+(rand()%(16*8)))*4+rand()%4;
        fontTex->dat[r]^=1<<(rand()%8);
        fontTex->flags|=TEX_FLAG_DIRTY;
    }
    for(i=0;i<num/100;i++)
    {
        int x=rand()%90;
        int y=rand()%40;
        tile *t=&map[x][y];
        switch(rand()%3)
        {
          case 0:
            t->r^=1<<(rand()%8);
            break;
          case 1:
            t->g^=1<<(rand()%8);;
            break;
          case 2:
            t->b^=1<<(rand()%8);
            break;
        }
        addChange(x,y);
    }
    for(i=0;i<num/1000;i++)
    {
        int x=rand()%90;
        int y=rand()%40;
        tile *t=&map[x][y];
        switch(rand()%4)
        {
          case 0:
            t->r^=1<<(rand()%8);
            break;
          case 1:
            t->g^=1<<(rand()%8);;
            break;
          case 2:
            t->b^=1<<(rand()%8);
            break;
          case 3:
            t->c^=1<<(rand()%8);
            break;
        }
        addChange(x,y);
    }
    for(i=0;i<num/10000;i++)
    {
        addChaosElemental();
    }
    for(i=0;i<num/100000;i++)
    {
        addSuperiorChaosElemental();
    }
}

void chaos_damage(creature *caster, int x, int y, int damage)
{
    if(x<0||y<0||x>=90||y>=40)
      return;
    if(map[x][y].occupant && (map[x][y].occupant!=caster || (rand()%20)==3))
      creature_damage(map[x][y].occupant,map[x][y].occupant->defense+rand()%damage,caster);
    int i;
    for(i=0;i<5;i++)
    {
        tile *t=&map[x][y];
        switch((rand()%61)/20)
        {
          case 0:
            t->r^=rand()%256;
            break;
          case 1:
            t->g^=rand()%256;
            break;
          case 2:
            t->b^=rand()%256;
            break;
          case 3:
            t->c=' '+rand()%(16*6);
            break;
        }
    }
    addChange(x,y);
}

int chaos_bolt(creature *this, int x, int y)
{
    if(this->power<1000)
    {
        creature_message(this,"You don't have enough magic.");
        return 0;
    }
    corrupt(rand()%500);
    this->wait=this->moveRate*5;
    this->power-=1000;
    int x2=this->x;
    int y2=this->y;
    int i;
    for(i=0;i<10;i++)
    {
        x2+=x;
        y2+=y;
        chaos_damage(this,x2,y2,this->level*2);
    }
    return 1;
}

int chaos_flee(creature *this)
{
    if(this->power<10000)
    {
        creature_message(this,"You don't have enough magic.");
        return 0;
    }
    regenMap();
    this->wait=this->moveRate*20;
    this->power-=10000;
    corrupt(rand()%5000);
    return 1;
}

int chaos_drain(creature *this)
{
    this->wait=this->moveRate*10;
    this->power+=rand()%5000;
    corrupt(1000+rand()%1000);
    return 1;
}


int chaos_blast(creature *this, int x, int y)
{
    if(this->power<500)
    {
        creature_message(this,"You don't have enough magic.");
        return 0;
    }
    corrupt(rand()%400);
    this->wait=this->moveRate*4;
    this->power-=500;
    int x2=this->x+x;
    int y2=this->y+y;
    chaos_damage(this,x2,y2,this->level*2.5);
    chaos_damage(this,x2,y2-1,this->level*1.8);
    chaos_damage(this,x2,y2+1,this->level*1.8);
    chaos_damage(this,x2-1,y2,this->level*1.8);
    chaos_damage(this,x2+1,y2,this->level*1.8);
    return 1;
}

int chaos_ascension(creature *this)
{
    if(this->power<1000000)
    {
        creature_message(this,"You don't have enough magic.");
        return 0;
    }
    this->defense+=rand()%100000;
    this->damage+=rand()%100000;
    this->defenseVar+=rand()%100000;
    this->damageVar+=rand()%100000;
    this->health+=rand()%100000;
    corrupt(100000+rand()%1000000);
    this->power-=1000000;
    return 1;
}

int chaos_speed(creature *this)
{
    if(this->power<5000)
    {
        creature_message(this,"You don't have enough magic.");
        return 0;
    }
    this->wait=this->moveRate*8;
    this->speed+=rand()%100;
    corrupt(100+rand()%1000);
    this->power-=5000;
    return 1;
}

int chaos_heal(creature *this)
{
    if(this->power<5000)
    {
        creature_message(this,"You don't have enough magic.");
        return 0;
    }
    this->wait=this->moveRate*8;
    this->health+=rand()%(this->level*5);
    corrupt(100+rand()%1000);
    this->power-=5000;
    return 1;
}

int chaos_barrier(creature *this)
{
    if(this->power<6000)
    {
        creature_message(this,"You don't have enough magic.");
        return 0;
    }
    this->wait=this->moveRate*7;
    this->defense+=rand()%(this->level*2);
    this->defenseVar+=rand()%(this->level);
    corrupt(100+rand()%1000);
    this->power-=6000;
    return 1;
}

int chaos_fury(creature *this)
{
    if(this->power<6000)
    {
        creature_message(this,"You don't have enough magic.");
        return 0;
    }
    this->wait=this->moveRate*7;
    this->damage+=rand()%(this->level*2);
    this->damageVar+=rand()%(this->level);
    corrupt(100+rand()%1000);
    this->power-=6000;
    return 1;
}

int chaos_gather(creature *this)
{
    if(this->power<2000)
    {
        creature_message(this,"You don't have enough magic.");
        return 0;
    }
    this->power-=2000;
    corrupt(100+rand()%1000);
    int x,y;
    for(y=0;y<40;y++)
      for(x=0;x<90;x++)
        if(map[x][y].item && (rand()%3)==0)
        {
            chaos_damage(this,x,y,rand()%10);
            creature_pickupItem(this,x,y);
        }
}

int chaos_death(creature *this)
{
    if(this->power<20000)
    {
        creature_message(this,"You don't have enough magic.");
        return 0;
    }
    this->power-=20000;
    corrupt(1000+rand()%10000);
    int x,y;
    for(y=0;y<40;y++)
      for(x=0;x<90;x++)
        if(map[x][y].occupant && (rand()%2) && map[x][y].occupant != this)
        {
            chaos_damage(this,x,y,this->level*5);
        }
}

int chaos_confusion(creature *this)
{
    if(this->power<10000)
    {
        creature_message(this,"You don't have enough magic.");
        return 0;
    }
    this->power-=10000;
    corrupt(500+rand()%5000);
    int x,y;
    for(y=0;y<40;y++)
      for(x=0;x<90;x++)
        if(map[x][y].occupant && (rand()%2) && map[x][y].occupant != this)
        {
            map[x][y].occupant->act=rand()&1?enemy_act:enemy_act_toPlayer;
        }
}

