#include "creature.h"
#include "map.h"
#include "item.h"


creature player;
void creature_message(creature *this, char *text)
{
    if(this==&player)
      addText(text);
}

void creature_pickupItem(creature *this, int x, int y)
{
    item *i=map[x][y].item;
    if(!i)
      return;
    switch(i->type)
    {
      case ITEM_GOLD:
        this->gold += i->quantity;
        map[x][y].item=0;
        free(i);
        break;
      case ITEM_ARROW:
        this->arrows += i->quantity;
        map[x][y].item=0;
        free(i);
        break;
      case ITEM_HEALTH:
        this->health += ((rand()%this->level)+1)*i->quantity;
        map[x][y].item=0;
        free(i);
        break;
      case ITEM_GEM:
        this->power += i->charge;
        map[x][y].item=0;
        free(i);
        break;
      case ITEM_ORDER:
        uncorrupt(this->gold);
        this->gold=0;
        map[x][y].item=0;
        free(i);
        break;
      
    }
    this->wait=this->moveRate/2;
}

void creature_pickup(creature *this)
{
    if(!map[this->x][this->y].item)
    {
        creature_message(this,"There is nothing to pick up here!");
        return;
    }
    creature_pickupItem(this,this->x,this->y);
}

char *creature_speakVerb(creature *this)
{
    int n=rand()%7;
    switch(n)
    {
      case 0:
        return "mutters";
      case 1:
        return "says";
      case 2:
        return "grumbles";
      case 3:
        return "grunts";
      case 4:
        return "wheezes";
      case 5:
        return "chuckles";
      case 6:
        return "hollers";
    }
}

creature *creature_new()
{
    creature *t = calloc(sizeof(struct creature),1);
    if(t)
    {
        player.prev->next = t;
        t->prev = player.prev;
        t->next = &player;
        player.prev = t;
        t->moveRate = 10000;
        t->level = 1;
        t->act = 0;
        t->bump = 0;
        t->health = 1;
        t->maxHealth = 1;
        t->damage = 1;
        t->damageVar = 1;
        t->arrows = 0;
    }
    return t;
}

void creature_sayTo(creature *this, creature *other, char *text)
{
    if(!text || !other || !this)
      return;
    creature_message(other,this->name);
    creature_message(other,creature_speakVerb(this));
    creature_message(other,text);
}

int creature_emptyCol(int col)
{
    int i,n=0;
    for(i=0;i<40;i++)
      if(!map[col][i].occupant)
        n++;
    return n;
}

int creature_emptyRow(int row)
{
    int i,n=0;
    for(i=0;i<90;i++)
      if(!map[i][row].occupant)
        n++;
    return n;
}

void creature_placeCol(creature *this, int col)
{
    int i,j=0;
    i=rand()%creature_emptyCol(col);
    while(i>0)
    {
        if(!map[col][j].occupant)
          i--;
        j++;
        if(j>=40)
          return;
    }
    map[col][j].occupant=this;
    this->x=col;
    this->y=j;
    addChange(col,j);
}

void creature_placeRow(creature *this, int row)
{
    int i=rand()%creature_emptyRow(row),j=0;
    while(i>0)
    {
        if(!map[j][row].occupant)
          i--;
        j++;
        if(j>=90)
          return;
    }
    map[j][row].occupant=this;
    this->x=j;
    this->y=row;
    addChange(j,row);
}

void creature_arrive(creature *arrival)
{
    if(!arrival)
      return;
    int i,dir=rand()&1;
    switch(rand()%2)
    {
        case 0:
          for(i=dir?0:89;dir?(i<90):(i>=0);dir?i++:i--)
            if(creature_emptyCol(i))
            {
                creature_placeCol(arrival,i);
                return;
            }
          break;
        case 1:
          for(i=dir?0:39;dir?(i<40):(i>=0);dir?i++:i--)
            if(creature_emptyRow(i))
            {
                creature_placeRow(arrival,i);
                return;
            }
          break;
    }
}

void creature_remove(creature *this)
{
    if(this==&player)
    {
        addText("You die");
        pause();
        end_game();
        return;
    }
    map[this->x][this->y].occupant=0;
    this->next->prev=this->prev;
    this->prev->next=this->next;
    free(this);
}

void creature_die(creature *this,creature *attacker)
{
    if(this==&player)
    {
        addText("You die");
        pause();
        end_game();
        return;
    }
    addText(this->name);
    addText("dies.");
    if(attacker==&player && (rand()%player.level)==0)
      player_levelup();
    map[this->x][this->y].occupant=0;
    if(map[this->x][this->y].r<240)
      map[this->x][this->y].r+=rand()%15;
    if(map[this->x][this->y].g>20)
      map[this->x][this->y].g-=rand()%15;
    if(map[this->x][this->y].b>20)
      map[this->x][this->y].b-=rand()%15;
    tileChanged(this->x,this->y);
    while(this->level>0)
    {
        dropNear(item_rand(3,5,1,this->level),this->x,this->y);
        this->level-=5;
    }
    dropGold(this->gold,this->x,this->y);
    dropArrows(this->arrows,this->x,this->y);
    this->next->prev=this->prev;
    this->prev->next=this->next;
    free(this);
}

int inRange(int x, int y, int x2, int y2, int range)
{
    return ((x-x2)*(x-x2)+(y-y2)*(y-y2)<=range*range);
}

void creature_noDamage(creature *this, creature *attacker)
{
    creature_message(this,"You are not hurt!");
    creature_message(attacker,"It is unaffected!");
}

void creature_heal(creature *this, int damage, creature *attacker)
{
}

void creature_damage(creature *this, int damage, creature *attacker)
{
    int defense=this->defense+(rand()%(this->defenseVar+1));
    if(damage<0)
      creature_heal(this,-damage,attacker);
    else if(damage<defense)
      creature_noDamage(this,attacker);
    else
    {
        this->health-=(damage-defense);
        creature_message(this,"You are hit!");
        creature_message(attacker,"You hit!");
        if(this->health<=0)
          creature_die(this,attacker);
    }
}

int creature_getDam(creature *this)
{
    return this->damage + (rand()%(this->damageVar+1));
}

int creature_attack(struct creature *this, struct creature *other)
{
    if(!this || !other)
      return;
    if(!inRange(this->x,this->y,other->x,other->y,2))
    {
        creature_message(this,"It is too far away to attack right now.");
        return 0;
    }
    creature_message(this,"You attack");
    creature_message(this,other->name);
    creature_message(this,".");
    creature_message(other,this->name);
    creature_message(other,"attacks you.");
    creature_damage(other,creature_getDam(this),this);
    this->wait=this->moveRate*2;
    return 1;
}

int creature_shoot(struct creature *this, int x, int y)
{
    if(!this || x<0 || y<0 || x>=90 || y>=40)
      return;
    if(!inRange(this->x,this->y,x,y,20))
    {
        creature_message(this,"It is too far away to attack right now.");
        return 0;
    }
    if(this->arrows<1)
    {
        creature_message(this,"You don't have any arrows.");
        return 0;
    }
    this->arrows--;
    creature *other=map[x][y].occupant;
    if(!other)
    {
        creature_message(this,"You shoot into empty space...");
        dropArrows(rand()%2,x,y);
        return 1;
    }
    creature_message(this,"You shoot");
    creature_message(this,other->name);
    creature_message(this,".");
    creature_message(other,this->name);
    creature_message(other,"shoots you.");
    other->arrows+=rand()%2;
    creature_damage(other,creature_getDam(this),this);
    this->wait=this->moveRate*2.5;
    return 1;
}

void bump_null(struct creature *this, struct creature *other)
{
}

void act_null(struct creature *this)
{
}

char *pileOf()
{
    switch(rand()%4)
    {
      case 0:
        return "pile of";
      case 1:
        return "heap of";
      case 2:
        return "stack of";
      case 3:
        return "bunch of";
    }
}

char numbuf[100]; 
char *num(int n)
{
    sprintf(numbuf,"%d",n);
    return numbuf;
}

void creature_move(creature *c, int x, int y)
{
    if(x==0 && y==0)
    {
        c->wait = c->moveRate;
        return;
    }
    if(c->x+x<0 || c->x+x>=90 || c->y+y<0 || c->y+y>=40)
      return;
    if(map[c->x+x][c->y+y].occupant != 0)
    {
        if(c->bump)
          c->bump(c,map[c->x+x][c->y+y].occupant);
        return;
    }
    if(map[c->x+x][c->y+y].item)
    {
        if(map[c->x+x][c->y+y].item->quantity>1)
        {
            if(map[c->x+x][c->y+y].item->nameType & ITEMNAME_STACK)
            {
                creature_message(c,"You see a");
                creature_message(c,pileOf());
            }
            else
              creature_message(c,"You see");
            creature_message(c,num(map[c->x+x][c->y+y].item->quantity));
            creature_message(c,map[c->x+x][c->y+y].item->name_plural);
            creature_message(c,"here");
        }
        else
        {
            if(map[c->x+x][c->y+y].item->nameType & ITEMNAME_VOWEL)
              creature_message(c,"You see an");
            else
              creature_message(c,"You see a");
            creature_message(c,map[c->x+x][c->y+y].item->name_singular);
            creature_message(c,"here");
        }
    }
    map[c->x][c->y].occupant = 0;
    tileChanged(c->x,c->y);
    c->x+=x;
    c->y+=y;
    map[c->x][c->y].occupant = c;
    tileChanged(c->x,c->y);
    c->wait = c->moveRate;
}
