#include "win.h"
#include "graphics.h"
#include "map.h"
#include <gl/gl.h>

void texBlock_update(texBlock *block)
{
    glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,block->width,block->height,0,GL_RGBA,GL_UNSIGNED_BYTE,block->dat);
    block->flags &= ~TEX_FLAG_DIRTY;
}

void drawPix(unsigned char *dest, unsigned char *src, unsigned char r, unsigned char g, unsigned char b)
{
    float f=src[3]/255.0;
    float f2=1-f;
    float r2=r/255.0,g2=g/255.0,b2=b/255.0;
    dest[0]=src[0]*f*r2+dest[0]*f2;
    dest[1]=src[1]*f*g2+dest[1]*f2;
    dest[2]=src[2]*f*b2+dest[2]*f2;
    dest[3]=src[3]*f+dest[3]*f2;
}

void drawCharToImage(unsigned char c, unsigned char r, unsigned char g, unsigned char b, int x, int y, texBlock *image, texBlock *dat)
{
    int i,j;
    int cX = c%16;
    int cY = 15-c/16;
    for(j=0;j<12;j++)
    {
        for(i=0;i<8;i++)
        {
            drawPix(&image->dat[(i+x+(j+y)*image->width)*4],&(dat->dat[(i+cX*8+(j+cY*12)*dat->width)*4]),r,g,b);
        }
    }
    image->flags|=TEX_FLAG_DIRTY;
}

void drawTexBlock(texBlock *block, int x, int y)
{
    if(!block)
      return;
    glColor3f(1,1,1);
    glBindTexture(GL_TEXTURE_2D,block->tex);
    if(block->flags & TEX_FLAG_DIRTY)
      texBlock_update(block);
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
    glBegin(GL_QUADS);
      glTexCoord2f(0,0);
      glVertex2i(x,y);
      glTexCoord2f(0,(float)block->Fheight/block->height);
      glVertex2i(x,y+block->Fheight);
      glTexCoord2f((float)block->Fwidth/block->width,(float)block->Fheight/block->height);
      glVertex2i(x+block->Fwidth,y+block->Fheight);
      glTexCoord2f((float)block->Fwidth/block->width,0);
      glVertex2i(x+block->Fwidth,y);
    glEnd();
}

void drawCharDirect(int x, int y, unsigned char c, unsigned char r, unsigned char g, unsigned char b, texBlock *block)
{
    glColor3ub(r,g,b);
    glBindTexture(GL_TEXTURE_2D,block->tex);
    if(block->flags & TEX_FLAG_DIRTY)
      texBlock_update(block);
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
    glBegin(GL_QUADS);
      glTexCoord2f((float)((c%16)*8)/block->width,(float)((15-c/16)*12)/block->height);
      glVertex2i(x,y);
      glTexCoord2f((float)((c%16)*8)/block->width,(float)((15-c/16)*12+12)/block->height);
      glVertex2i(x,y+12);
      glTexCoord2f((float)((c%16)*8+8)/block->width,(float)((15-c/16)*12+12)/block->height);
      glVertex2i(x+8,y+12);
      glTexCoord2f((float)((c%16)*8+8)/block->width,(float)((15-c/16)*12)/block->height);
      glVertex2i(x+8,y);
    glEnd();
}

int p2(int n)
{
    int i=1;
    while(i<n)
      i*=2;
    return i;
}

texBlock* texBlock_new(int w, int h)
{
    if(w<1 || h<1)
      return NULL;
    texBlock *temp = calloc(sizeof(struct texBlock),1);
    if(!temp)
      return NULL;
    temp->Fwidth = w;
    temp->Fheight = h;
    w=p2(w);
    h=p2(h);
    temp->dat = calloc(4*w*h,1);
    if(!temp->dat)
    {
        free(temp);
        return NULL;
    }
    int i;
    for(i=0;i<w*h;i++)
      temp->dat[i*4+3]=255;
    temp->width = w;
    temp->height = h;
    temp->flags = TEX_FLAG_DIRTY;
    glGenTextures(1,&(temp->tex));
    glBindTexture(GL_TEXTURE_2D,temp->tex);
    glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,temp->width,temp->height,0,GL_RGBA,GL_BYTE,temp->dat);
    return temp;
}

texBlock* texBlock_clone(texBlock *src)
{
    if(!src || !src->dat)
      return NULL;
    texBlock *temp = calloc(sizeof(struct texBlock),1);
    if(!temp)
      return NULL;
    temp->dat = calloc(4*src->width*src->height,1);
    if(!temp->dat)
    {
        free(temp);
        return NULL;
    }
    temp->width=src->width;
    temp->height=src->height;
    temp->Fwidth=src->Fwidth;
    temp->Fheight=src->Fheight;
    temp->flags = TEX_FLAG_DIRTY;
    memcpy(temp->dat,src->dat,4*src->width*src->height);
    glGenTextures(1,&(temp->tex));
    glBindTexture(GL_TEXTURE_2D,temp->tex);
    glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,temp->width,temp->height,0,GL_RGBA,GL_BYTE,temp->dat);
    return temp;
}
