#include <stdlib.h>
#include "item.h"
#include "map.h"

item *item_new()
{
    item *temp = calloc(sizeof(struct item),1);
    if(!temp)
      return NULL;
    temp->r=255;
    temp->g=255;
    temp->b=255;
    temp->c='?';
    temp->next=NULL;
    temp->prev=NULL;
    temp->type=0;
    temp->charge=0;
    temp->damage=0;
    temp->damageVar=0;
    temp->defense=0;
    temp->defenseVar=0;
    temp->quantity=1;
    temp->nameType=ITEMNAME_VOWEL|ITEMNAME_STACK;
    temp->name_singular="item";
    temp->name_plural="items";
    return temp;
}

item *item_leveled(int level)
{
    item *i=NULL;
    if(level>2)
      level=2;
    switch(rand()%100?rand()%(level*2):4)
    {
      case 0:
        i=item_new();
        i->type=ITEM_GOLD;
        i->b=0;
        i->c='$';
        i->nameType=0;
        i->name_singular="gold piece";
        i->name_plural="gold pieces";
        i->quantity=1+(rand()%(level+5))*level;
        break;
      case 1:
        i=item_new();
        i->type=ITEM_ARROW;
        i->r=200+rand()%50;
        i->g=100+rand()%25;
        i->b=0;
        i->c=170;
        i->nameType=ITEMNAME_STACK | ITEMNAME_VOWEL;
        i->name_singular="arrow";
        i->name_plural="arrows";
        i->quantity=1+(rand()%(level+5))*level;
        break;
      case 2:
        i=item_new();
        i->type=ITEM_HEALTH;
        i->r=200+rand()%50;
        i->g=100+rand()%25;
        i->b=200+rand()%50;
        i->c=3;
        i->nameType=ITEMNAME_STACK;
        i->name_singular="health";
        i->name_plural="health";
        i->quantity=1+(rand()%(level+1))*level;
        break;
      case 3:
        i=item_new();
        i->type=ITEM_GEM;
        i->r=100+(rand()%100);
        i->g=100+(rand()%100);
        i->b=100+(rand()%100);
        i->c=15;
        i->nameType=ITEMNAME_STACK;
        i->name_singular="magical gem";
        i->name_plural="magical gems";
        i->charge = 100+rand()%(level*100+50);
        break;
      case 4:
        i=item_new();
        i->type=ITEM_ORDER;
        i->r=255;
        i->g=255;
        i->b=255;
        i->c=168;
        i->nameType=0;//ITEMNAME_STACK;
        i->name_singular="scroll of order";
        i->name_plural="scrolls of order";
        i->quantity=1;
        break;
      default:
        i=item_new();
        break;
    }
    return i;
}

item *item_rand(int probNumerator, int probDenominator, int minLevel, int maxLevel)
{
    if(probDenominator>0 && (rand()%probDenominator)<probNumerator && maxLevel>=minLevel)
    {
        return item_leveled(minLevel+(rand()%((maxLevel-minLevel)+1)));
    }
    return NULL;
}

int itemSpace(int x, int y, int r)
{
    int minX=x-r,minY=y-r,maxX=x+r,maxY=y+r;
    if(minX<0)
      minX=0;
    if(minY<0)
      minY=0;
    if(maxX>=90)
      maxX=89;
    if(maxY>=40)
      maxY=39;
    int i,j;
    for(j=minY;j<=maxY;j++)
      for(i=minX;i<=maxX;i++)
        if(!map[i][j].item)
          return 1;
    return 0;
}

void dropInItemSpace(item *it, int x, int y, int r)
{
    int minX=x-r,minY=y-r,maxX=x+r,maxY=y+r;
    if(minX<0)
      minX=0;
    if(minY<0)
      minY=0;
    if(maxX>=90)
      maxX=89;
    if(maxY>=40)
      maxY=39;
    int i,j;
    for(j=minY;j<=maxY;j++)
      for(i=minX;i<=maxX;i++)
        if(!map[i][j].item)
        {
            map[i][j].item=it;
            tileChanged(i,j);
            return;
        }
}

void dropNear(item *i, int x, int y)
{
    int r=0;
    while(r<20)
    {
        if(itemSpace(x,y,r))
        {
            dropInItemSpace(i,x,y,r);
            return;
        }
        r++;
    }
}

void dropGold(int gold, int x, int y)
{
    if(gold>0)
    {
        item *i=item_new();
        if(i)
        {
            i->type=ITEM_GOLD;
            i->b=0;
            i->c='$';
            i->nameType=0;
            i->name_singular="gold piece";
            i->name_plural="gold pieces";
            i->quantity=gold;
            dropNear(i,x,y);
        }
    }
}

void dropArrows(int num, int x, int y)
{
    if(num>0)
    {
        item *i=item_new();
        if(i)
        {
            i->type=ITEM_ARROW;
            i->r=200+rand()%50;
            i->g=100+rand()%25;
            i->b=0;
            i->c=170;
            i->nameType=ITEMNAME_STACK | ITEMNAME_VOWEL;
            i->name_singular="arrow";
            i->name_plural="arrows";
            i->quantity=num;
            dropNear(i,x,y);
        }
    }
}
