#include <windows.h>
#include <stdio.h>
#include <time.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include "win.h"
#include "graphics.h"
#include "logic.h"
#include "text.h"

FILE *errout;
RECT windowRect;
int quit=0;
int quitMessage=0;
HDC hdc;
HWND hwnd_globl;
int drawing = 0;
int closing = 0;
HANDLE logicThread;

void outputError(char *str)
{
    int err;
    char *errStr=0;
    if(!str)
      str="Untitled error";
    err=GetLastError();
    if(FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS,NULL,err,LANG_USER_DEFAULT,(LPTSTR)&errStr,0,NULL)==0)
      fprintf(errout,"Failure to display error message for \'%s\'.\nRegistry error code %d\nError display error code %d\n",str,err,(int)GetLastError());
    else
      fprintf(errout,"%s.\nError code %d:\n%s\n",str,err,errStr);
}

texBlock* loadTexture(FILE *f)
{
    if(!f)
      return NULL;
    BITMAPFILEHEADER bfh;
    BITMAPINFOHEADER bih;
    fread(&bfh,sizeof(BITMAPFILEHEADER),1,f);
    fread(&bih,sizeof(BITMAPINFOHEADER),1,f);
    fseek(f,bfh.bfOffBits,SEEK_SET);
    int width=bih.biWidth;
    int height=bih.biHeight;
    texBlock* t=texBlock_new(width, height);
    if(!t)
      return NULL;
    int x,y;
    for(y=0;y<height;y++)
    {
        for(x=0;x<width;x++)
        {
            t->dat[(y*width+x)*4+2]=fgetc(f);
            t->dat[(y*width+x)*4+1]=fgetc(f);
            t->dat[(y*width+x)*4]=fgetc(f);
            if(t->dat[(y*width+x)*4]==255&&t->dat[(y*width+x)*4+1]==0&&t->dat[(y*width+x)*4+2]==255)
            {
                t->dat[(y*width+x)*4]=t->dat[(y*width+x)*4+1]=t->dat[(y*width+x)*4+2]=t->dat[(y*width+x)*4+3]=0;
            }
            else t->dat[(y*width+x)*4+3]=255;
        }
    }
    glGenTextures(1,&(t->tex));
    glBindTexture(GL_TEXTURE_2D,t->tex);
    glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,t->width,t->height,0,GL_RGBA,GL_UNSIGNED_BYTE,t->dat);
    return t;
}

unsigned char keys[256];

LRESULT CALLBACK winProc(HWND hwnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
    static HGLRC hglrc;
    switch(uMsg)
    {
      case WM_CREATE:
      {
        int iPixelFormat;
        PIXELFORMATDESCRIPTOR pfd=
            {sizeof(PIXELFORMATDESCRIPTOR),1,PFD_DRAW_TO_WINDOW|PFD_SUPPORT_OPENGL|PFD_DOUBLEBUFFER,
            PFD_TYPE_RGBA,24,
            0,0,0,0,0,0,           /*//Not used*/
            0,
            0,                     /*//Not used*/
            0,
            0,0,0,0,               /*//Not used*/
            32,0,0,PFD_MAIN_PLANE,
            0,0,0,0};              /*//Not used*/
        hdc=GetDC(hwnd);
        iPixelFormat = ChoosePixelFormat(hdc,&pfd);
        if(!iPixelFormat)
        {
            outputError("Failure to ChoosePixelFormat");
            quit=1;
            break;
        }
        if(!SetPixelFormat(hdc, iPixelFormat, &pfd))
        {
            outputError("Failure to SetPixelFormat");
            quit=1;
            break;
        }
        hglrc=wglCreateContext(hdc);
        if(!hglrc)
        {
            outputError("Failure to create glrc");
            quit=1;
            break;
        }
        if(!wglMakeCurrent(hdc,hglrc))
        {
            outputError("Failure to make glrc current");
            quit=1;
            wglDeleteContext(hglrc);
            break;
        }
        break;
      }
      case WM_SIZE:
        break;
      case WM_MOVE:
        break;
      case WM_MOVING:
      case WM_SIZING:
        break;
      case WM_DESTROY:
        closing=1;
        quitMessage=0;
        wglDeleteContext(hglrc);
        break;
      case WM_QUIT:
        closing=1;
        quitMessage=wParam;
        wglDeleteContext(hglrc);
        break;
      case WM_KEYDOWN:
      {
          static unsigned char out[2];
          GetKeyboardState(keys);
          if(ToAscii(wParam,MapVirtualKey(wParam,0),keys,(LPWORD)out,0)==1)
          {
              char key=out[0];
              input(key);
          }
          else
          {
              if(wParam==VK_LEFT)
                input('4');
              else if(wParam==VK_RIGHT)
                input('6');
              else if(wParam==VK_UP)
                input('8');
              else if(wParam==VK_DOWN)
                input('2');
          }
          break;
      }
      case WM_PAINT:
      default:
        break;
    }
    return DefWindowProc(hwnd,uMsg,wParam,lParam);
}

int logicThreadRunning=0;
DWORD WINAPI startGameLoop(LPVOID lpParameter)
{
    logicThreadRunning=1;
    gameLoop();
    logicThreadRunning=0;
}

int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
    MSG msg;
    int t;
    DWORD dwStyle;
    DWORD dwExStyle;
    WNDCLASSEX windowClass;
    hPrevInstance=NULL;
    printf("%s\n",lpCmdLine);
    errout=fopen("err.txt","w");
    srand(time(NULL));
    windowRect.top=100;
    windowRect.left=100;
    windowRect.bottom=640;
    windowRect.right=820;
    dwStyle=WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME;
    dwExStyle=0;//WS_EX_ACCEPTFILES;
    windowClass.cbSize=sizeof(WNDCLASSEX);
    windowClass.style=CS_DBLCLKS|CS_HREDRAW|CS_VREDRAW;
    windowClass.lpfnWndProc=winProc;
    windowClass.cbClsExtra=0;
    windowClass.cbWndExtra=0;
    windowClass.hInstance=hInstance;
    windowClass.hIcon=NULL;
    windowClass.hCursor=LoadCursor(NULL,IDC_ARROW);
    windowClass.hbrBackground=(HBRUSH)(COLOR_BACKGROUND+1);
    windowClass.lpszMenuName=NULL;
    windowClass.lpszClassName="MWB_window";
    windowClass.hIconSm=NULL;
    if(AdjustWindowRectEx(&windowRect,dwStyle,0,dwExStyle)==0)
    {
        outputError("Failure to adjust window rect");
        return 1;
    }
    if(RegisterClassEx(&windowClass)==0)
    {
        outputError("Failure to register window class");
        return 1;
    }
    hwnd_globl=CreateWindowEx(dwExStyle,windowClass.lpszClassName,"7DRL",dwStyle,windowRect.left,windowRect.top,windowRect.right-windowRect.left,windowRect.bottom-windowRect.top,NULL,NULL,hInstance,NULL);
    if(!hwnd_globl)
    {
        outputError("Failure to create window");
        return 1;
    }
    ShowWindow(hwnd_globl,nCmdShow);
    UpdateWindow(hwnd_globl);
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glEnable(GL_BLEND);
    glEnable(GL_TEXTURE_2D);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0,720,0,540,0,65536);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    init();
    if(closing)
      return 1;
    logicThread=CreateThread(0,0,startGameLoop,0,0,0);
    if(!logicThread)
    {
        outputError("could not create logic thread");
        return 1;
    }
    t=clock();
    while(!closing)
    {
        while(PeekMessage(&msg,hwnd_globl,0,0,PM_REMOVE))
        {
            TranslateMessage(&msg); 
            DispatchMessage(&msg);
            if(closing)
              break;
        }
        if(closing)
          break;
        update();
        if(drawing==0)
        {
            drawing=1;
            //glClear(GL_COLOR_BUFFER_BIT);
            render();
            SwapBuffers(hdc);
            drawing=0;
        }
        if(50-(clock()-t)>0)
          Sleep(50-(clock()-t));
        t=clock();
    }
    while(ResumeThread(logicThread));
    int c=0;
    while(mi)
    {
        c++;
        Sleep(100);
        if(c>100)
        {
            TerminateThread(miH,1);
            break;
        }
    }
    c=0;
    while(logicThreadRunning)
    {
        c++;
        Sleep(100);
        if(c>100)
        {
            TerminateThread(logicThread,1);
            break;
        }
    }
    return quitMessage;
}
